/*! \file   include/direct-button.h
    \brief  query button states
    \author Markus L. Noga <noga@inrialpes.fr>
*/
    
/*
 *  The contents of this file are subject to the Mozilla Public License
 *  Version 1.0 (the "License"); you may not use this file except in
 *  compliance with the License. You may obtain a copy of the License at
 *  http://www.mozilla.org/MPL/
 *
 *  Software distributed under the License is distributed on an "AS IS"
 *  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 *  License for the specific language governing rights and limitations
 *  under the License.
 *
 *  The Original Code is legOS code, released March 19, 1999.
 *
 *  The Initial Developer of the Original Code is Markus L. Noga.
 *  Portions created by Markus L. Noga are Copyright (C) 1999
 *  Markus L. Noga. All Rights Reserved.
 *
 *  Contributor(s): Markus L. Noga <noga@inrialpes.fr>
 */

#ifndef __direct_button_h__
#define __direct_button_h__


///////////////////////////////////////////////////////////////////////////////
//
// Definitions
//
///////////////////////////////////////////////////////////////////////////////

//
// the button bitmasks
// buttons at @0xb7:8 and @0xbe:8
//

#define	BUTTON_ONOFF	0x0002		// 0x02 @ 0xffb7
#define BUTTON_RUN	0x0004		// 0x04 @ 0xffb7
#define BUTTON_VIEW	0x4000		// 0x40 @ 0xffbe
#define BUTTON_PROGRAM	0x8000		// 0x80 @ 0xffbe

//
// buttons are active low
//

//
// true if any of the specified buttons is released
//
#define RELEASED(state,button)	((state) & (button)) 

//
// true if all of the specified buttons are pressed
//
#define PRESSED(state,button)	(!RELEASED(state,button))


///////////////////////////////////////////////////////////////////////////////
//
// Functions
//
///////////////////////////////////////////////////////////////////////////////

//
// get button states
// masks defined above
//
extern inline int button_state(void) {
	int rv;
__asm__ __volatile__(
	"mov.b @0xb7:8,%0l\n"
	"mov.b @0xbe:8,%0h\n"
	: "=r" (rv)				// output
	: 					// input
	: "cc"					// clobbered
	);
	return rv;
}


#endif
